package edu.mit.csail.pag.recrash.tracer.item;

import java.util.Hashtable;
import java.util.Set;

import checkers.quals.Default;
import checkers.quals.Nullable;

/**
 * This is a trace item per method. A trace item includes a method info and Item
 * will be stored and passed to the server
 * 
 * @author hunkim
 * 
 */
@Default("checkers.quals.NonNull")
public class MethodTraceItem {

	// store static objects
	@Nullable
	private Hashtable<String, String> serializedStaticObjects = null;

	// argument type array in order
	ClonedArgumentObject serializedArgumentObject[];

	// Method name
	String methodName;

	int nounceNumber;

	/**
	 * Get method name, arguments, and type names and initialize the method.
	 * 
	 * @param methodName
	 * @param objects
	 * @param types
	 * @param doCloneObject
	 * @param nounceNumber
	 */
	public MethodTraceItem(String methodName, Object objects[], String types[],
			boolean[] doCloneObject, int nounceNumber) {
		this.methodName = methodName;
		this.nounceNumber = nounceNumber;

		// create the array in the same size of the argNumber + 1 ( for this)
		serializedArgumentObject = new ClonedArgumentObject[objects.length];

		// walk into each object and type and add them in the array
		for (int i = 0; i < objects.length; i++) {
			addObject(i, objects[i], types[i], doCloneObject[i]);
		}
	}

	/**
	 * Get total number of arguments
	 * 
	 * @return
	 */
	public int getNumberOfArguments() {
		return this.serializedArgumentObject.length;
	}

	/**
	 * Argument name
	 * 
	 * @param index
	 * @return
	 */
	public ClonedArgumentObject getArgumentObject(int index) {
		assert (index >= 0 && index < getNumberOfArguments());
		return this.serializedArgumentObject[index];
	}

	/**
	 * Get method name
	 * 
	 * @return
	 */
	public String getMethodName() {
		return methodName;
	}

	/**
	 * Get method name
	 * 
	 * @return
	 */
	public String getShortMethodName() {
		int lastPoint = methodName.lastIndexOf('.');
		if (lastPoint == -1) {
			return methodName;
		}

		@Nullable
		String ret = methodName.substring(lastPoint + 1);
		assert (ret != null);

		return ret;
	}

	/**
	 * 
	 * @param index
	 *            index of the object
	 * @param object
	 *            object to add
	 * @param classTypeName
	 *            type name of the object
	 * @param needToBeCloned
	 *            indicate if it is need to be serialized
	 */
	private void addObject(int index, Object object, String classTypeName,
			boolean needToBeCloned) {
		serializedArgumentObject[index] = new ClonedArgumentObject(object,
				classTypeName, needToBeCloned);
	}

	/**
	 * Add one static object
	 * 
	 * @param name
	 * @param serializedXMLString
	 */
	public void addStatic(String name, String serializedXMLString) {
		if (serializedStaticObjects == null) {
			serializedStaticObjects = new Hashtable<String, String>();
		}
		serializedStaticObjects.put(name, serializedXMLString);
	}

	/**
	 * Read static object by full classname
	 * 
	 * @param name
	 *            argument name
	 * @return
	 */
	public @Nullable
	String readStatic(String name) {
		if (serializedStaticObjects == null) {
			return null;
		}
		return serializedStaticObjects.get(name);
	}

	/**
	 * get list of static classes stored in the trace
	 * 
	 * @return
	 */
	public @Nullable
	Set<String> getStaticClassNameSet() {
		if (serializedStaticObjects == null) {
			return null;
		}
		return serializedStaticObjects.keySet();
	}

	/**
	 * Checking only nounceNumber is enough?
	 * 
	 * @param nounceNumber
	 * @return
	 */
	public boolean equals(int nounceNumber) {
		return nounceNumber == this.nounceNumber;
	}

	@Override
	public int hashCode() {
		return methodName.hashCode() + nounceNumber;
	}
}
